]> git.sesse.net Git - cubemap/blobdiff - server.h
Update metacube2.h with the latest version (sync with Nageru).
[cubemap] / server.h
index 7297fd7797daf3adaf73738da93eb861fca89ad6..1e6f44a57eb8e7a07382c1e7b473db80f97899aa 100644 (file)
--- a/server.h
+++ b/server.h
 #include <string>
 #include <vector>
 
+#include "tlse.h"
+
 #include "client.h"
 #include "stream.h"
 #include "thread.h"
 
+class Acceptor;
 class ClientProto;
 struct Stream;
 
@@ -47,21 +50,23 @@ public:
        // These will be deferred until the next time an iteration in do_work() happens,
        // and the order between them are undefined.
        // XXX: header should ideally be ordered with respect to data.
-       void add_client_deferred(int sock);
-       void add_data_deferred(int stream_index, const char *data, size_t bytes, StreamStartSuitability suitable_for_stream_start);
+       void add_client_deferred(int sock, Acceptor *acceptor);
+       void add_data_deferred(int stream_index, const char *data, size_t bytes, uint16_t metacube_flags);
 
        // These should not be called while running, since that would violate
        // threading assumptions (ie., that epoll is only called from one thread
        // at the same time).
        CubemapStateProto serialize();
        void add_client_from_serialized(const ClientProto &client);
-       int add_stream(const std::string &url, size_t bytes_received, size_t prebuffering_bytes, Stream::Encoding encoding);
+       int add_stream(const std::string &url, size_t bytes_received, size_t prebuffering_bytes, Stream::Encoding encoding, Stream::Encoding src_encoding);
        int add_stream_from_serialized(const StreamProto &stream, int data_fd);
        int lookup_stream_by_url(const std::string &url) const;
        void set_backlog_size(int stream_index, size_t new_size);
        void set_prebuffering_bytes(int stream_index, size_t new_amount);
        void set_encoding(int stream_index, Stream::Encoding encoding);
-       void add_ping(const std::string &url, const std::string &allow_origin);
+       void set_src_encoding(int stream_index, Stream::Encoding encoding);
+       void add_gen204(const std::string &url, const std::string &allow_origin);
+       void create_tls_context_for_acceptor(const Acceptor *acceptor);
 
 private:
        // Mutex protecting queued_add_clients.
@@ -79,7 +84,7 @@ private:
        //    can be taken a lot of the time.
        //      
        // Protected by <queued_clients_mutex>.
-       std::vector<int> queued_add_clients;
+       std::vector<std::pair<int, Acceptor *> > queued_add_clients;
 
        // All variables below this line are protected by the mutex.
        mutable pthread_mutex_t mutex;
@@ -113,6 +118,9 @@ private:
        int epoll_fd;
        epoll_event events[EPOLL_MAX_EVENTS];
 
+       // For each TLS-enabled acceptor, our private server context for its key pair.
+       std::map<const Acceptor *, TLSContext *> tls_server_contexts;
+
        // The actual worker thread.
        virtual void do_work();
 
@@ -131,11 +139,29 @@ private:
        // but it's cheaper than taking it in and out all the time.
        void process_client(Client *client);
 
+       // If the TLS library wants to write anything to this client,
+       // output it. Returns true if the processing should go to sleep
+       // (an error, or lack of outgoing buffer space).
+       bool send_pending_tls_data(Client *client);
+
+       // Reads regular data fro ma socket. Returns -1 if the processing
+       // should go to sleep (an error, or no data available yet), otherwise
+       // the number of bytes read.
+       int read_nontls_data(Client *client, char *buf, size_t max_size);
+
+       // Reads (decrypted) data from a TLS socket. Returns -1 if the processing
+       // should go to sleep (an error, or no data available yet), otherwise
+       // the number of bytes read. The buffer will be used as scratch space
+       // for TLS data, so it can be overwritten by more bytes than what is returned.
+       int read_tls_data(Client *client, char *buf, size_t max_size);
+
        // Close a given client socket, and clean up after it.
        void close_client(Client *client);
 
-       // Parse the HTTP request. Returns a HTTP status code (200/400/404),
-       // or -200 for a pong (which should be answered with 200).
+       // Listen for a different set of epoll events.
+       void change_epoll_events(Client *client, uint32_t events);
+
+       // Parse the HTTP request. Returns a HTTP status code (200/204/400/404).
        int parse_request(Client *client);
 
        // Construct the HTTP header, and set the client into
@@ -146,13 +172,13 @@ private:
        // the SENDING_SHORT_RESPONSE state.
        void construct_error(Client *client, int error_code);
 
-       // Construct a pong, and set the client into the SENDING_SHORT_RESPONSE state.
-       void construct_pong(Client *client);
+       // Construct a 204, and set the client into the SENDING_SHORT_RESPONSE state.
+       void construct_204(Client *client);
 
        void process_queued_data();
        void skip_lost_data(Client *client);
 
-       void add_client(int sock);
+       void add_client(int sock, Acceptor *acceptor);
 };
 
 #endif  // !defined(_SERVER_H)